package com.example.demo;

import java.util.Objects;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;

@SpringBootApplication
public class DemoApplication implements CommandLineRunner {
	private final Logger log = LoggerFactory.getLogger(DemoApplication.class);

	private final TypeService typeService;
	private final AgeRatingService ageRatingService;
	private final MovieService movieService;

	public DemoApplication(TypeService typeService, AgeRatingService ageRatingService, MovieService movieService) {
		this.typeService = typeService;
		this.ageRatingService = ageRatingService;
		this.movieService = movieService;
	}

	public static void main(String[] args) {
		SpringApplication.run(DemoApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		if (args.length > 0 && Objects.equals("--populate", args[0])) {
			log.info("Create default types values");
			final var type1 = typeService.create(new TypeEntity(null, "Фильм"));
			final var type2 = typeService.create(new TypeEntity(null, "Сериал"));

			log.info("Create default ageRatings values");
			// final var ageRating1 = ageRatingService.create(new AgeRatingEntity(null,
			// "0+"));
			// final var ageRating2 = ageRatingService.create(new AgeRatingEntity(null,
			// "6+"));
			// final var ageRating3 = ageRatingService.create(new AgeRatingEntity(null,
			// "12+"));
			final var ageRating4 = ageRatingService.create(new AgeRatingEntity(null, "16+"));
			final var ageRating5 = ageRatingService.create(new AgeRatingEntity(null, "18+"));

			log.info("Create default movies values");
			movieService.create(new MovieEntity(null, "Бойцовский клуб", type1, false,
					"src/assets/main-page/posters/PeakyBlinders.png",
					"Сотрудник страховой компании страдает хронической бессонницей и отчаянно пытается вырваться из мучительно скучной жизни...",
					1999, "США", "Интриги. Хаос. Мыло", "Дэвид Финчер", ageRating5,
					"https://www.youtube.com/embed/dfeUzm6KF4g?si=U2D-2WNsJzkcxlBX"));
			movieService.create(new MovieEntity(null, "Американский психопат", type1, true,
					"src/assets/main-page/posters/AmericanPsycho.jpg",
					"Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
					2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating5,
					"https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));
			movieService.create(new MovieEntity(null, "Острые козырьки", type2, false,
					"src/assets/main-page/posters/PeakyBlinders.png",
					"Сериал рассказывает историю восхождения одной ирландской семьи...",
					2013, "Великобритания", "The streets are theirs", "Энтони Бирн, Колм МакКарти", ageRating4,
					"https://www.youtube.com/embed/7cOfn_sjlXM?si=fxNoZjlUon1SZd70"));
		}
	}
}
