package com.example.demo.movies.model;

import java.util.Objects;

import com.example.demo.core.model.BaseEntity;
import com.example.demo.types.model.TypeEntity;

import jakarta.persistence.Column;
import jakarta.persistence.Entity;
import jakarta.persistence.JoinColumn;
import jakarta.persistence.JoinTable;
import jakarta.persistence.ManyToOne;
import jakarta.persistence.Table;

import com.example.demo.ageRatings.model.AgeRatingEntity;

@Entity
@Table(name = "movies")
public class MovieEntity extends BaseEntity {
    @Column(nullable = false, unique = true, length = 30)
    private String title;

    @ManyToOne
    @JoinColumn(name = "typeId", nullable = false)
    private TypeEntity type;

    @ManyToOne
    @JoinColumn(name = "ageRatingId", nullable = false)
    private AgeRatingEntity ageRating;

    @Column(nullable = false)
    private boolean requiresSubscription;

    @Column(length = 50)
    private String poster;

    @Column(nullable = false, unique = true, length = 100)
    private String description;

    private Integer releaseDate;

    @Column(nullable = false, unique = true, length = 20)
    private String country;

    @Column(nullable = false, unique = true, length = 30)
    private String tagline;

    @Column(nullable = false, unique = true, length = 30)
    private String director;

    @Column(nullable = false, unique = true, length = 100)
    private String video;

    public MovieEntity() {
    }

    public MovieEntity(Long id, String title, TypeEntity type, boolean requiresSubscription, String poster,
            String description, Integer releaseDate, String country, String tagline, String director,
            AgeRatingEntity ageRating, String video) {
        this.title = title;
        this.type = type;
        this.requiresSubscription = requiresSubscription;
        this.poster = poster;
        this.description = description;
        this.releaseDate = releaseDate;
        this.country = country;
        this.tagline = tagline;
        this.director = director;
        this.ageRating = ageRating;
        this.video = video;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public TypeEntity getType() {
        return type;
    }

    public void setType(TypeEntity type) {
        this.type = type;
    }

    public boolean getRequiresSubscription() {
        return requiresSubscription;
    }

    public void setRequiresSubscription(boolean requiresSubscription) {
        this.requiresSubscription = requiresSubscription;
    }

    public String getPoster() {
        return poster;
    }

    public void setPoster(String poster) {
        this.poster = poster;
    }

    public String getDescription() {
        return description;
    }

    public void setDescription(String description) {
        this.description = description;
    }

    public Integer getReleaseDate() {
        return releaseDate;
    }

    public void setReleaseDate(Integer releaseDate) {
        this.releaseDate = releaseDate;
    }

    public String getCountry() {
        return country;
    }

    public void setCountry(String country) {
        this.country = country;
    }

    public String getTagline() {
        return tagline;
    }

    public void setTagline(String tagline) {
        this.tagline = tagline;
    }

    public String getDirector() {
        return director;
    }

    public void setDirector(String director) {
        this.director = director;
    }

    public AgeRatingEntity getAgeRating() {
        return ageRating;
    }

    public void setAgeRating(AgeRatingEntity ageRating) {
        this.ageRating = ageRating;
    }

    public String getVideo() {
        return video;
    }

    public void setVideo(String video) {
        this.video = video;
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, title, type, requiresSubscription, poster, description, releaseDate, country, tagline,
                director, ageRating, video);
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null || getClass() != obj.getClass())
            return false;
        final MovieEntity other = (MovieEntity) obj;
        return Objects.equals(other.getId(), id)
                && Objects.equals(other.getTitle(), title)
                && Objects.equals(other.getType(), type)
                && Objects.equals(other.getRequiresSubscription(), requiresSubscription)
                && Objects.equals(other.getPoster(), poster)
                && Objects.equals(other.getDescription(), description)
                && Objects.equals(other.getReleaseDate(), releaseDate)
                && Objects.equals(other.getCountry(), country)
                && Objects.equals(other.getTagline(), tagline)
                && Objects.equals(other.getDirector(), director)
                && Objects.equals(other.getAgeRating(), ageRating)
                && Objects.equals(other.getVideo(), video);
    }
}
