package com.example.demo.users.service;

import java.util.List;
import java.util.Optional;

import org.springframework.stereotype.Service;

import com.example.demo.users.model.UserEntity;
import com.example.demo.users.repository.UserRepository;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;

@Service
public class UserService {
    private final UserRepository repository;
    private final MovieService movieService;

    public UserService(UserRepository repository, MovieService movieService) {
        this.movieService = movieService;
        this.repository = repository;
    }

    public List<UserEntity> getAll() {
        return repository.getAll();
    }

    public UserEntity get(Long id) {
        return Optional.ofNullable(repository.get(id))
                .orElseThrow(() -> new NotFoundException(id));
    }

    public UserEntity create(UserEntity entity) {
        return repository.create(entity);
    }

    public UserEntity update(Long id, UserEntity entity) {
        final UserEntity existsEntity = get(id);
        existsEntity.setEmail(entity.getEmail());
        return repository.update(existsEntity);
    }

    public UserEntity delete(Long id) {
        final UserEntity existsEntity = get(id);
        return repository.delete(existsEntity);
    }

    public void deleteAll() {
        repository.deleteAll();
    }

    public UserEntity addMovie(Long id, Long movieId) {
        final UserEntity existsEntity = get(id);

        MovieEntity movie = movieService.get(movieId);

        if (!existsEntity.getWatchLater().contains(movie.getId())) {
            existsEntity.getWatchLater().add(movie.getId());
        }

        return existsEntity;
    }
}
