package com.example.demo;

import java.util.ArrayList;
import java.util.Date;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import com.example.demo.ageRatings.model.AgeRatingEntity;
import com.example.demo.ageRatings.service.AgeRatingService;
import com.example.demo.core.error.NotFoundException;
import com.example.demo.movies.model.MovieEntity;
import com.example.demo.movies.service.MovieService;
import com.example.demo.types.model.TypeEntity;
import com.example.demo.types.service.TypeService;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class UserServiceTests {
	@Autowired
	private TypeService typeService;

	@Autowired
	private AgeRatingService ageRatingService;

	@Autowired
	private MovieService movieService;

	@Autowired
	private UserService userService;

	@Test
	void getTest() {
		Assertions.assertThrows(NotFoundException.class, () -> userService.get(0L));
	}

	@Test
	@Order(1)
	void createTest() {
		ageRatingService.deleteAll();
		typeService.deleteAll();
		movieService.deleteAll();
		userService.deleteAll();

		userService.create(new UserEntity(null, "myemail@mail.com", "12345678", new Date(), new ArrayList<>()));
		userService.create(new UserEntity(null, "secondemail@mail.com", "qwwerty", new Date(), new ArrayList<>()));
		final var last = userService
				.create(new UserEntity(null, "thirdemail@mail.com", "ytrewq", new Date(), new ArrayList<>()));

		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertEquals(last, userService.get(3L));
	}

	@Test
	@Order(2)
	void updateTest() {
		final var type1 = typeService.create(new TypeEntity(null, "Фильм"));
		final var ageRating5 = ageRatingService.create(new AgeRatingEntity(null, "18+"));

		final var movie1 = movieService.create(new MovieEntity(null, "Бойцовский клуб", type1, false,
				"src/assets/main-page/posters/PeakyBlinders.png",
				"Сотрудник страховой компании страдает хронической бессонницей и отчаянно пытается вырваться из мучительно скучной жизни...",
				1999, "США", "Интриги. Хаос. Мыло", "Дэвид Финчер", ageRating5,
				"https://www.youtube.com/embed/dfeUzm6KF4g?si=U2D-2WNsJzkcxlBX"));
		final var movie2 = movieService.create(new MovieEntity(null, "Американский психопат", type1, true,
				"src/assets/main-page/posters/AmericanPsycho.jpg",
				"Днем он ничем не отличается от окружающих, и в толпе вы не обратите на него внимания...",
				2000, "США, Канада", "Респектабельная внешность дьявола", "Мэри Хэррон", ageRating5,
				"https://www.youtube.com/embed/PpUWjff_OcM?si=e7tSm4V6qYkz406B"));

		userService.addMovie(1L, movie1.getId());
		final UserEntity newEntity = userService.addMovie(1L, movie2.getId());

		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertEquals(newEntity, userService.get(1L));
		Assertions.assertEquals(2, userService.get(1L).getWatchLater().size());

		userService.addMovie(1L, movie1.getId());
		Assertions.assertEquals(2, userService.get(1L).getWatchLater().size());
	}

	@Test
	@Order(3)
	void deleteTest() {
		userService.delete(3L);
		Assertions.assertEquals(2, userService.getAll().size());
		final UserEntity last = userService.get(2L);
		Assertions.assertEquals(2L, last.getId());

		final var newEntity = userService
				.create(new UserEntity(null, "thirdemail@mail.com", "ytrewq", new Date(), new ArrayList<>()));
		Assertions.assertEquals(3, userService.getAll().size());
		Assertions.assertEquals(4L, newEntity.getId());
	}
}
